{-# OPTIONS_GHC -fno-warn-unused-binds -fno-warn-missing-signatures #-}
{-# LANGUAGE CPP #-}
{-# LINE 1 "Lexer.x" #-}

module Lexer where

#if __GLASGOW_HASKELL__ >= 603
#include "ghcconfig.h"
#elif defined(__GLASGOW_HASKELL__)
#include "config.h"
#endif
#if __GLASGOW_HASKELL__ >= 503
import Data.Array
#else
import Array
#endif
{-# LINE 1 "templates/wrappers.hs" #-}
-- -----------------------------------------------------------------------------
-- Alex wrapper code.
--
-- This code is in the PUBLIC DOMAIN; you may copy it freely and use
-- it for any purpose whatsoever.





import Data.Word (Word8)
















import Data.Char (ord)
import qualified Data.Bits

-- | Encode a Haskell String to a list of Word8 values, in UTF8 format.
utf8Encode :: Char -> [Word8]
utf8Encode = uncurry (:) . utf8Encode'

utf8Encode' :: Char -> (Word8, [Word8])
utf8Encode' c = case go (ord c) of
                  (x, xs) -> (fromIntegral x, map fromIntegral xs)
 where
  go oc
   | oc <= 0x7f       = ( oc
                        , [
                        ])

   | oc <= 0x7ff      = ( 0xc0 + (oc `Data.Bits.shiftR` 6)
                        , [0x80 + oc Data.Bits..&. 0x3f
                        ])

   | oc <= 0xffff     = ( 0xe0 + (oc `Data.Bits.shiftR` 12)
                        , [0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ])
   | otherwise        = ( 0xf0 + (oc `Data.Bits.shiftR` 18)
                        , [0x80 + ((oc `Data.Bits.shiftR` 12) Data.Bits..&. 0x3f)
                        , 0x80 + ((oc `Data.Bits.shiftR` 6) Data.Bits..&. 0x3f)
                        , 0x80 + oc Data.Bits..&. 0x3f
                        ])



type Byte = Word8

-- -----------------------------------------------------------------------------
-- The input type
















































































-- -----------------------------------------------------------------------------
-- Token positions

-- `Posn' records the location of a token in the input text.  It has three
-- fields: the address (number of chacaters preceding the token), line number
-- and column of a token within the file. `start_pos' gives the position of the
-- start of the file and `eof_pos' a standard encoding for the end of file.
-- `move_pos' calculates the new position after traversing a given character,
-- assuming the usual eight character tab stops.














-- -----------------------------------------------------------------------------
-- Monad (default and with ByteString input)

























































































































































-- -----------------------------------------------------------------------------
-- Basic wrapper


type AlexInput = (Char,[Byte],String)

alexInputPrevChar :: AlexInput -> Char
alexInputPrevChar (c,_,_) = c

-- alexScanTokens :: String -> [token]
alexScanTokens str = go ('\n',[],str)
  where go inp__@(_,_bs,s) =
          case alexScan inp__ 0 of
                AlexEOF -> []
                AlexError _ -> error "lexical error"
                AlexSkip  inp__' _ln     -> go inp__'
                AlexToken inp__' len act -> act (take len s) : go inp__'

alexGetByte :: AlexInput -> Maybe (Byte,AlexInput)
alexGetByte (c,(b:bs),s) = Just (b,(c,bs,s))
alexGetByte (_,[],[])    = Nothing
alexGetByte (_,[],(c:s)) = case utf8Encode' c of
                             (b, bs) -> Just (b, (c, bs, s))



-- -----------------------------------------------------------------------------
-- Basic wrapper, ByteString version
































-- -----------------------------------------------------------------------------
-- Posn wrapper

-- Adds text positions to the basic model.













-- -----------------------------------------------------------------------------
-- Posn wrapper, ByteString version














-- -----------------------------------------------------------------------------
-- GScan wrapper

-- For compatibility with previous versions of Alex, and because we can.














alex_tab_size :: Int
alex_tab_size = 8
alex_base :: Array Int Int
alex_base = listArray (0 :: Int, 110)
  [ -8
  , -55
  , -54
  , -73
  , 0
  , 40
  , 0
  , 168
  , 296
  , 552
  , 488
  , 0
  , -1
  , 734
  , 706
  , 951
  , 1035
  , 1119
  , 1203
  , 1287
  , 1371
  , 1455
  , 1539
  , 1623
  , 1707
  , 0
  , -32
  , 0
  , 0
  , 1791
  , 1875
  , -47
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 1959
  , 2043
  , 2127
  , 2211
  , 2295
  , 2379
  , 2463
  , 0
  , 2547
  , 2631
  , 2715
  , 2799
  , 2883
  , 2967
  , 3051
  , 3135
  , 3219
  , 3303
  , 3387
  , 3471
  , 3555
  , 3639
  , 3723
  , 3807
  , 3891
  , 3975
  , 4059
  , 4143
  , 4227
  , 4311
  , 4395
  , 4479
  , 4563
  , 4647
  , 4731
  , 4815
  , 4899
  , 4983
  , 5067
  , 5151
  , 5235
  , 5319
  , 5403
  , 5487
  , 5571
  , 5655
  , 5739
  , 5823
  , 5907
  , 5991
  , 6075
  , 6159
  , 6243
  , 6327
  , 6411
  , 6495
  , 6579
  , 6663
  , 6747
  , 6831
  , 6915
  , 6999
  , 7083
  , 7167
  , 7251
  , 7335
  , 7419
  , 7503
  , 7587
  , 7671
  , 781
  ]

alex_table :: Array Int Int
alex_table = listArray (0 :: Int, 7926)
  [ 0
  , 12
  , 12
  , 12
  , 12
  , 12
  , 25
  , 32
  , 12
  , 12
  , 12
  , 12
  , 12
  , 13
  , 32
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 12
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 12
  , 36
  , 37
  , 28
  , 27
  , 47
  , 26
  , 0
  , 27
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 1
  , 33
  , 31
  , 2
  , 31
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 34
  , 0
  , 35
  , 0
  , 0
  , 0
  , 97
  , 87
  , 80
  , 55
  , 96
  , 89
  , 83
  , 83
  , 109
  , 83
  , 83
  , 83
  , 82
  , 67
  , 104
  , 91
  , 83
  , 73
  , 77
  , 94
  , 83
  , 72
  , 88
  , 83
  , 83
  , 83
  , 38
  , 0
  , 39
  , 10
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 3
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 8
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 9
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 110
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 9
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 11
  , 10
  , 8
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 4
  , 3
  , 7
  , 6
  , 6
  , 6
  , 5
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 14
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 15
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 18
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 19
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 20
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 23
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 24
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 42
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 45
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 48
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 49
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 51
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 52
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 108
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 53
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 107
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 106
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 56
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 57
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 103
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 58
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 101
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 59
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 60
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 61
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 98
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 66
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 69
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 70
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 95
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 84
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 92
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 86
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 85
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 81
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 90
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 79
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 78
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 76
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 93
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 75
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 74
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 71
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 68
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 63
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 64
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 62
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 100
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 65
  , 83
  , 102
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 105
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 54
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 50
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 46
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 44
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 41
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 40
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 30
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 29
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 22
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 21
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 17
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 83
  , 0
  , 83
  , 83
  , 83
  , 83
  , 83
  , 16
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 99
  , 83
  , 83
  , 83
  , 83
  , 43
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 83
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  , 0
  ]

alex_check :: Array Int Int
alex_check = listArray (0 :: Int, 7926)
  [ -1
  , 9
  , 10
  , 11
  , 12
  , 13
  , 61
  , 61
  , 9
  , 10
  , 11
  , 12
  , 13
  , 45
  , 61
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 32
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 32
  , 40
  , 41
  , 42
  , 43
  , 44
  , 45
  , -1
  , 47
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , 59
  , 60
  , 61
  , 62
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , -1
  , 93
  , -1
  , -1
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 123
  , -1
  , 125
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 0
  , 1
  , 2
  , 3
  , 4
  , 5
  , 6
  , 7
  , 8
  , 9
  , 10
  , 11
  , 12
  , 13
  , 14
  , 15
  , 16
  , 17
  , 18
  , 19
  , 20
  , 21
  , 22
  , 23
  , 24
  , 25
  , 26
  , 27
  , 28
  , 29
  , 30
  , 31
  , 32
  , 33
  , 34
  , 35
  , 36
  , 37
  , 38
  , 39
  , 40
  , 41
  , 42
  , 43
  , 44
  , 45
  , 46
  , 47
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , 58
  , 59
  , 60
  , 61
  , 62
  , 63
  , 64
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , 91
  , 92
  , 93
  , 94
  , 95
  , 96
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 123
  , 124
  , 125
  , 126
  , 127
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 10
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 128
  , 129
  , 130
  , 131
  , 132
  , 133
  , 134
  , 135
  , 136
  , 137
  , 138
  , 139
  , 140
  , 141
  , 142
  , 143
  , 144
  , 145
  , 146
  , 147
  , 148
  , 149
  , 150
  , 151
  , 152
  , 153
  , 154
  , 155
  , 156
  , 157
  , 158
  , 159
  , 160
  , 161
  , 162
  , 163
  , 164
  , 165
  , 166
  , 167
  , 168
  , 169
  , 170
  , 171
  , 172
  , 173
  , 174
  , 175
  , 176
  , 177
  , 178
  , 179
  , 180
  , 181
  , 182
  , 183
  , 184
  , 185
  , 186
  , 187
  , 188
  , 189
  , 190
  , 191
  , 192
  , 193
  , 194
  , 195
  , 196
  , 197
  , 198
  , 199
  , 200
  , 201
  , 202
  , 203
  , 204
  , 205
  , 206
  , 207
  , 208
  , 209
  , 210
  , 211
  , 212
  , 213
  , 214
  , 215
  , 216
  , 217
  , 218
  , 219
  , 220
  , 221
  , 222
  , 223
  , 224
  , 225
  , 226
  , 227
  , 228
  , 229
  , 230
  , 231
  , 232
  , 233
  , 234
  , 235
  , 236
  , 237
  , 238
  , 239
  , 240
  , 241
  , 242
  , 243
  , 244
  , 245
  , 246
  , 247
  , 248
  , 249
  , 250
  , 251
  , 252
  , 253
  , 254
  , 255
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , 39
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 48
  , 49
  , 50
  , 51
  , 52
  , 53
  , 54
  , 55
  , 56
  , 57
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , 65
  , 66
  , 67
  , 68
  , 69
  , 70
  , 71
  , 72
  , 73
  , 74
  , 75
  , 76
  , 77
  , 78
  , 79
  , 80
  , 81
  , 82
  , 83
  , 84
  , 85
  , 86
  , 87
  , 88
  , 89
  , 90
  , -1
  , -1
  , -1
  , -1
  , 95
  , -1
  , 97
  , 98
  , 99
  , 100
  , 101
  , 102
  , 103
  , 104
  , 105
  , 106
  , 107
  , 108
  , 109
  , 110
  , 111
  , 112
  , 113
  , 114
  , 115
  , 116
  , 117
  , 118
  , 119
  , 120
  , 121
  , 122
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  ]

alex_deflt :: Array Int Int
alex_deflt = listArray (0 :: Int, 110)
  [ -1
  , -1
  , -1
  , 11
  , 11
  , 4
  , 4
  , -1
  , -1
  , 13
  , 13
  , 13
  , -1
  , 13
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  , -1
  ]

alex_accept = listArray (0 :: Int, 110)
  [ AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccNone
  , AlexAccSkip
  , AlexAccSkip
  , AlexAcc 96
  , AlexAcc 95
  , AlexAcc 94
  , AlexAcc 93
  , AlexAcc 92
  , AlexAcc 91
  , AlexAcc 90
  , AlexAcc 89
  , AlexAcc 88
  , AlexAcc 87
  , AlexAcc 86
  , AlexAcc 85
  , AlexAcc 84
  , AlexAcc 83
  , AlexAcc 82
  , AlexAcc 81
  , AlexAcc 80
  , AlexAcc 79
  , AlexAcc 78
  , AlexAcc 77
  , AlexAcc 76
  , AlexAcc 75
  , AlexAcc 74
  , AlexAcc 73
  , AlexAcc 72
  , AlexAcc 71
  , AlexAcc 70
  , AlexAcc 69
  , AlexAcc 68
  , AlexAcc 67
  , AlexAcc 66
  , AlexAcc 65
  , AlexAcc 64
  , AlexAcc 63
  , AlexAcc 62
  , AlexAcc 61
  , AlexAcc 60
  , AlexAcc 59
  , AlexAcc 58
  , AlexAcc 57
  , AlexAcc 56
  , AlexAcc 55
  , AlexAcc 54
  , AlexAcc 53
  , AlexAcc 52
  , AlexAcc 51
  , AlexAcc 50
  , AlexAcc 49
  , AlexAcc 48
  , AlexAcc 47
  , AlexAcc 46
  , AlexAcc 45
  , AlexAcc 44
  , AlexAcc 43
  , AlexAcc 42
  , AlexAcc 41
  , AlexAcc 40
  , AlexAcc 39
  , AlexAcc 38
  , AlexAcc 37
  , AlexAcc 36
  , AlexAcc 35
  , AlexAcc 34
  , AlexAcc 33
  , AlexAcc 32
  , AlexAcc 31
  , AlexAcc 30
  , AlexAcc 29
  , AlexAcc 28
  , AlexAcc 27
  , AlexAcc 26
  , AlexAcc 25
  , AlexAcc 24
  , AlexAcc 23
  , AlexAcc 22
  , AlexAcc 21
  , AlexAcc 20
  , AlexAcc 19
  , AlexAcc 18
  , AlexAcc 17
  , AlexAcc 16
  , AlexAcc 15
  , AlexAcc 14
  , AlexAcc 13
  , AlexAcc 12
  , AlexAcc 11
  , AlexAcc 10
  , AlexAcc 9
  , AlexAcc 8
  , AlexAcc 7
  , AlexAcc 6
  , AlexAcc 5
  , AlexAcc 4
  , AlexAcc 3
  , AlexAcc 2
  , AlexAcc 1
  , AlexAcc 0
  ]

alex_actions = array (0 :: Int, 97)
  [ (96,alex_action_2)
  , (95,alex_action_3)
  , (94,alex_action_4)
  , (93,alex_action_5)
  , (92,alex_action_6)
  , (91,alex_action_7)
  , (90,alex_action_8)
  , (89,alex_action_9)
  , (88,alex_action_10)
  , (87,alex_action_11)
  , (86,alex_action_12)
  , (85,alex_action_13)
  , (84,alex_action_14)
  , (83,alex_action_14)
  , (82,alex_action_15)
  , (81,alex_action_16)
  , (80,alex_action_17)
  , (79,alex_action_18)
  , (78,alex_action_18)
  , (77,alex_action_19)
  , (76,alex_action_20)
  , (75,alex_action_21)
  , (74,alex_action_22)
  , (73,alex_action_23)
  , (72,alex_action_24)
  , (71,alex_action_25)
  , (70,alex_action_26)
  , (69,alex_action_27)
  , (68,alex_action_28)
  , (67,alex_action_29)
  , (66,alex_action_30)
  , (65,alex_action_31)
  , (64,alex_action_32)
  , (63,alex_action_33)
  , (62,alex_action_34)
  , (61,alex_action_35)
  , (60,alex_action_36)
  , (59,alex_action_37)
  , (58,alex_action_37)
  , (57,alex_action_37)
  , (56,alex_action_37)
  , (55,alex_action_37)
  , (54,alex_action_37)
  , (53,alex_action_37)
  , (52,alex_action_37)
  , (51,alex_action_37)
  , (50,alex_action_37)
  , (49,alex_action_37)
  , (48,alex_action_37)
  , (47,alex_action_37)
  , (46,alex_action_37)
  , (45,alex_action_37)
  , (44,alex_action_37)
  , (43,alex_action_37)
  , (42,alex_action_37)
  , (41,alex_action_37)
  , (40,alex_action_37)
  , (39,alex_action_37)
  , (38,alex_action_37)
  , (37,alex_action_37)
  , (36,alex_action_37)
  , (35,alex_action_37)
  , (34,alex_action_37)
  , (33,alex_action_37)
  , (32,alex_action_37)
  , (31,alex_action_37)
  , (30,alex_action_37)
  , (29,alex_action_37)
  , (28,alex_action_37)
  , (27,alex_action_37)
  , (26,alex_action_37)
  , (25,alex_action_37)
  , (24,alex_action_37)
  , (23,alex_action_37)
  , (22,alex_action_37)
  , (21,alex_action_37)
  , (20,alex_action_37)
  , (19,alex_action_37)
  , (18,alex_action_37)
  , (17,alex_action_37)
  , (16,alex_action_37)
  , (15,alex_action_37)
  , (14,alex_action_37)
  , (13,alex_action_37)
  , (12,alex_action_37)
  , (11,alex_action_37)
  , (10,alex_action_37)
  , (9,alex_action_37)
  , (8,alex_action_37)
  , (7,alex_action_37)
  , (6,alex_action_37)
  , (5,alex_action_37)
  , (4,alex_action_37)
  , (3,alex_action_37)
  , (2,alex_action_37)
  , (1,alex_action_37)
  , (0,alex_action_38)
  ]

{-# LINE 61 "Lexer.x" #-}

-- Each action has type :: String -> Token

-- The token type:
data Token  = TIdent String
            | TInt Int
            | TBool Bool
            | TIf
            | TThen
            | TElse
            | TWhile
            | TDo
            | TSkip
            | TNot
            | TAssign
            | TArithmeticOp String
            | TStar
            | TBoolOp String
            | TRelOp String
            | TSemicolon
            | TBlockOpen
            | TBlockClose
            | TParenOpen
            | TParenClose
            | TBraceOpen
            | TBraceClose
            | TBegin
            | TEnd
            | TProc
            | TIs
            | TVal
            | TRes
            | TCall
            | TComma
            | TMalloc
            | TFree
            | TContinue
            | TBreak
            | TTyInt
            deriving (Eq,Show)

alex :: String -> [Token]
alex = alexScanTokens

alex_action_2 =  \s -> TBool True 
alex_action_3 =  \s -> TBool False 
alex_action_4 =  \s -> TIf 
alex_action_5 =  \s -> TThen 
alex_action_6 =  \s -> TElse 
alex_action_7 =  \s -> TWhile 
alex_action_8 =  \s -> TDo 
alex_action_9 =  \s -> TSkip 
alex_action_10 =  \s -> TNot 
alex_action_11 =  \s -> TContinue
alex_action_12 =  \s -> TBreak
alex_action_13 =  \s -> TAssign 
alex_action_14 =  \s -> TArithmeticOp s 
alex_action_15 =  \s -> TStar 
alex_action_16 =  \s -> TBoolOp s 
alex_action_17 =  \s -> TBoolOp s 
alex_action_18 =  \s -> TRelOp s 
alex_action_19 =  \s -> TSemicolon 
alex_action_20 =  \s -> TBlockOpen 
alex_action_21 =  \s -> TBlockClose 
alex_action_22 =  \s -> TParenOpen 
alex_action_23 =  \s -> TParenClose 
alex_action_24 =  \s -> TBraceOpen 
alex_action_25 =  \s -> TBraceClose 
alex_action_26 =  \s -> TBegin 
alex_action_27 =  \s -> TEnd 
alex_action_28 =  \s -> TProc 
alex_action_29 =  \s -> TIs 
alex_action_30 =  \s -> TVal 
alex_action_31 =  \s -> TRes 
alex_action_32 =  \s -> TCall 
alex_action_33 =  \s -> TComma 
alex_action_34 =  \s -> TMalloc 
alex_action_35 =  \s -> TFree 
alex_action_36 =  \s -> TTyInt 
alex_action_37 =  \s -> TIdent s 
alex_action_38 =  \s -> TInt (read s) 
{-# LINE 1 "templates/GenericTemplate.hs" #-}
-- -----------------------------------------------------------------------------
-- ALEX TEMPLATE
--
-- This code is in the PUBLIC DOMAIN; you may copy it freely and use
-- it for any purpose whatsoever.

-- -----------------------------------------------------------------------------
-- INTERNALS and main scanner engine




































































alexIndexInt16OffAddr arr off = arr ! off
























alexIndexInt32OffAddr arr off = arr ! off











quickIndex arr i = arr ! i


-- -----------------------------------------------------------------------------
-- Main lexing routines

data AlexReturn a
  = AlexEOF
  | AlexError  !AlexInput
  | AlexSkip   !AlexInput !Int
  | AlexToken  !AlexInput !Int a

-- alexScan :: AlexInput -> StartCode -> AlexReturn a
alexScan input__ (sc)
  = alexScanUser undefined input__ (sc)

alexScanUser user__ input__ (sc)
  = case alex_scan_tkn user__ input__ (0) input__ sc AlexNone of
  (AlexNone, input__') ->
    case alexGetByte input__ of
      Nothing ->



                                   AlexEOF
      Just _ ->



                                   AlexError input__'

  (AlexLastSkip input__'' len, _) ->



    AlexSkip input__'' len

  (AlexLastAcc k input__''' len, _) ->



    AlexToken input__''' len (alex_actions ! k)


-- Push the input through the DFA, remembering the most recent accepting
-- state it encountered.

alex_scan_tkn user__ orig_input len input__ s last_acc =
  input__ `seq` -- strict in the input
  let
  new_acc = (check_accs (alex_accept `quickIndex` (s)))
  in
  new_acc `seq`
  case alexGetByte input__ of
     Nothing -> (new_acc, input__)
     Just (c, new_input) ->



      case fromIntegral c of { (ord_c) ->
        let
                base   = alexIndexInt32OffAddr alex_base s
                offset = (base + ord_c)
                check  = alexIndexInt16OffAddr alex_check offset

                new_s = if (offset >= (0)) && (check == ord_c)
                          then alexIndexInt16OffAddr alex_table offset
                          else alexIndexInt16OffAddr alex_deflt s
        in
        case new_s of
            (-1) -> (new_acc, input__)
                -- on an error, we want to keep the input *before* the
                -- character that failed, not after.
            _ -> alex_scan_tkn user__ orig_input (if c < 0x80 || c >= 0xC0 then (len + (1)) else len)
                                                -- note that the length is increased ONLY if this is the 1st byte in a char encoding)
                        new_input new_s new_acc
      }
  where
        check_accs (AlexAccNone) = last_acc
        check_accs (AlexAcc a  ) = AlexLastAcc a input__ (len)
        check_accs (AlexAccSkip) = AlexLastSkip  input__ (len)

        check_accs (AlexAccPred a predx rest)
           | predx user__ orig_input (len) input__
           = AlexLastAcc a input__ (len)
           | otherwise
           = check_accs rest
        check_accs (AlexAccSkipPred predx rest)
           | predx user__ orig_input (len) input__
           = AlexLastSkip input__ (len)
           | otherwise
           = check_accs rest


data AlexLastAcc
  = AlexNone
  | AlexLastAcc !Int !AlexInput !Int
  | AlexLastSkip     !AlexInput !Int

data AlexAcc user
  = AlexAccNone
  | AlexAcc Int
  | AlexAccSkip

  | AlexAccPred Int (AlexAccPred user) (AlexAcc user)
  | AlexAccSkipPred (AlexAccPred user) (AlexAcc user)

type AlexAccPred user = user -> AlexInput -> Int -> AlexInput -> Bool

-- -----------------------------------------------------------------------------
-- Predicates on a rule

alexAndPred p1 p2 user__ in1 len in2
  = p1 user__ in1 len in2 && p2 user__ in1 len in2

--alexPrevCharIsPred :: Char -> AlexAccPred _
alexPrevCharIs c _ input__ _ _ = c == alexInputPrevChar input__

alexPrevCharMatches f _ input__ _ _ = f (alexInputPrevChar input__)

--alexPrevCharIsOneOfPred :: Array Char Bool -> AlexAccPred _
alexPrevCharIsOneOf arr _ input__ _ _ = arr ! alexInputPrevChar input__

--alexRightContext :: Int -> AlexAccPred _
alexRightContext (sc) user__ _ _ input__ =
     case alex_scan_tkn user__ input__ (0) input__ sc AlexNone of
          (AlexNone, _) -> False
          _ -> True
        -- TODO: there's no need to find the longest
        -- match when checking the right context, just
        -- the first match will do.

